@@ -69,12 +69,13 @@
 //! // our router
 //! let app = Router::new()
 //!     .route("/", get(root))
-//!     .route("/foo", get(foo))
+//!     .route("/foo", get(get_foo).post(post_foo))
 //!     .route("/foo/bar", get(foo_bar));
 //!
 //! // which calls one of these handlers
 //! async fn root() {}
-//! async fn foo() {}
+//! async fn get_foo() {}
+//! async fn post_foo() {}
 //! async fn foo_bar() {}
 //! # async {
 //! # axum::Server::bind(&"".parse().unwrap()).serve(app.into_make_service()).await.unwrap();
@@ -207,7 +208,7 @@
 //! tower = "<latest-version>"
 //! ```
 //!
-//! The `"full"` feature for hyper and tokio isn't strictly necessary but its
+//! The `"full"` feature for hyper and tokio isn't strictly necessary but it's
 //! the easiest way to get started.
 //!
 //! Note that [`hyper::Server`] is re-exported by axum so if thats all you need
@@ -325,7 +326,7 @@ pub mod response;
 pub mod routing;
 
 #[cfg(test)]
-mod tests;
+mod test_helpers;
 
 pub use add_extension::{AddExtension, AddExtensionLayer};
 #[doc(no_inline)]
