@@ -12,7 +12,13 @@ macro_rules! opaque_future {
         pin_project_lite::pin_project! {
             $(#[$m])*
             pub struct $name<$($param),*> {
-                #[pin] pub(crate) future: $actual,
+                #[pin] future: $actual,
+            }
+        }
+
+        impl<$($param),*> $name<$($param),*> {
+            pub(crate) fn new(future: $actual) -> Self {
+                Self { future }
             }
         }
 
@@ -27,8 +33,12 @@ macro_rules! opaque_future {
             $actual: std::future::Future,
         {
             type Output = <$actual as std::future::Future>::Output;
+
             #[inline]
-            fn poll(self: std::pin::Pin<&mut Self>, cx: &mut std::task::Context<'_>) -> std::task::Poll<Self::Output> {
+            fn poll(
+                self: std::pin::Pin<&mut Self>,
+                cx: &mut std::task::Context<'_>,
+            ) -> std::task::Poll<Self::Output> {
                 self.project().future.poll(cx)
             }
         }
