@@ -1,7 +1,6 @@
 //! Routing between [`Service`]s and handlers.
 
-use self::future::RouterFuture;
-use self::not_found::NotFound;
+use self::{future::RouterFuture, not_found::NotFound};
 use crate::{
     body::{box_body, Body, BoxBody},
     extract::{
@@ -37,6 +36,9 @@ mod not_found;
 mod route;
 mod strip_prefix;
 
+#[cfg(test)]
+mod tests;
+
 pub use self::{
     into_make_service::IntoMakeService, method_filter::MethodFilter,
     method_not_allowed::MethodNotAllowed, route::Route,
@@ -59,10 +61,12 @@ impl RouteId {
 }
 
 /// The router type for composing handlers and services.
+#[derive(Debug)]
 pub struct Router<B = Body> {
     routes: HashMap<RouteId, Route<B>>,
     node: Node,
     fallback: Fallback<B>,
+    nested_at_root: bool,
 }
 
 impl<B> Clone for Router<B> {
@@ -71,6 +75,7 @@ impl<B> Clone for Router<B> {
             routes: self.routes.clone(),
             node: self.node.clone(),
             fallback: self.fallback.clone(),
+            nested_at_root: self.nested_at_root,
         }
     }
 }
@@ -84,16 +89,6 @@ where
     }
 }
 
-impl<B> fmt::Debug for Router<B> {
-    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
-        f.debug_struct("Router")
-            .field("routes", &self.routes)
-            .field("node", &self.node)
-            .field("fallback", &self.fallback)
-            .finish()
-    }
-}
-
 pub(crate) const NEST_TAIL_PARAM: &str = "axum_nest";
 const NEST_TAIL_PARAM_CAPTURE: &str = "/*axum_nest";
 
@@ -110,6 +105,7 @@ where
             routes: Default::default(),
             node: Default::default(),
             fallback: Fallback::Default(Route::new(NotFound)),
+            nested_at_root: false,
         }
     }
 
@@ -136,7 +132,7 @@ where
         let id = RouteId::next();
 
         if let Err(err) = self.node.insert(path, id) {
-            panic!("Invalid route: {}", err);
+            self.panic_on_matchit_error(err);
         }
 
         self.routes.insert(id, Route::new(service));
@@ -163,6 +159,10 @@ where
 
         let prefix = path;
 
+        if path == "/" {
+            self.nested_at_root = true;
+        }
+
         match try_downcast::<Router<B>, _>(svc) {
             // if the user is nesting a `Router` we can implement nesting
             // by simplying copying all the routes and adding the prefix in
@@ -173,6 +173,10 @@ where
                     node,
                     // discard the fallback of the nested router
                     fallback: _,
+                    // nesting a router that has something nested at root
+                    // doesn't mean something is nested at root in _this_ router
+                    // thus we don't need to propagate that
+                    nested_at_root: _,
                 } = router;
 
                 for (id, nested_path) in node.paths {
@@ -208,10 +212,11 @@ where
             routes,
             node,
             fallback,
+            nested_at_root,
         } = other;
 
         if let Err(err) = self.node.merge(node) {
-            panic!("Invalid route: {}", err);
+            self.panic_on_matchit_error(err);
         }
 
         for (id, route) in routes {
@@ -225,23 +230,22 @@ where
             (Fallback::Custom(_), pick @ Fallback::Custom(_)) => pick,
         };
 
+        self.nested_at_root = self.nested_at_root || nested_at_root;
+
         self
     }
 
     #[doc = include_str!("../docs/routing/layer.md")]
-    pub fn layer<L, LayeredReqBody, LayeredResBody>(self, layer: L) -> Router<LayeredReqBody>
+    pub fn layer<L, NewReqBody, NewResBody>(self, layer: L) -> Router<NewReqBody>
     where
         L: Layer<Route<B>>,
-        L::Service: Service<
-                Request<LayeredReqBody>,
-                Response = Response<LayeredResBody>,
-                Error = Infallible,
-            > + Clone
+        L::Service: Service<Request<NewReqBody>, Response = Response<NewResBody>, Error = Infallible>
+            + Clone
             + Send
             + 'static,
-        <L::Service as Service<Request<LayeredReqBody>>>::Future: Send + 'static,
-        LayeredResBody: http_body::Body<Data = Bytes> + Send + 'static,
-        LayeredResBody::Error: Into<BoxError>,
+        <L::Service as Service<Request<NewReqBody>>>::Future: Send + 'static,
+        NewResBody: http_body::Body<Data = Bytes> + Send + 'static,
+        NewResBody::Error: Into<BoxError>,
     {
         let layer = ServiceBuilder::new()
             .layer_fn(Route::new)
@@ -255,7 +259,7 @@ where
                 let route = Layer::layer(&layer, route);
                 (id, route)
             })
-            .collect::<HashMap<RouteId, Route<LayeredReqBody>>>();
+            .collect();
 
         let fallback = self.fallback.map(|svc| Layer::layer(&layer, svc));
 
@@ -263,6 +267,41 @@ where
             routes,
             node: self.node,
             fallback,
+            nested_at_root: self.nested_at_root,
+        }
+    }
+
+    #[doc = include_str!("../docs/routing/route_layer.md")]
+    pub fn route_layer<L, NewResBody>(self, layer: L) -> Self
+    where
+        L: Layer<Route<B>>,
+        L::Service: Service<Request<B>, Response = Response<NewResBody>, Error = Infallible>
+            + Clone
+            + Send
+            + 'static,
+        <L::Service as Service<Request<B>>>::Future: Send + 'static,
+        NewResBody: http_body::Body<Data = Bytes> + Send + 'static,
+        NewResBody::Error: Into<BoxError>,
+    {
+        let layer = ServiceBuilder::new()
+            .layer_fn(Route::new)
+            .layer(MapResponseBodyLayer::new(box_body))
+            .layer(layer);
+
+        let routes = self
+            .routes
+            .into_iter()
+            .map(|(id, route)| {
+                let route = Layer::layer(&layer, route);
+                (id, route)
+            })
+            .collect();
+
+        Router {
+            routes,
+            node: self.node,
+            fallback: self.fallback,
+            nested_at_root: self.nested_at_root,
         }
     }
 
@@ -279,7 +318,7 @@ where
         self
     }
 
-    /// Convert this router into a [`MakeService`], that is a [`Service`] who's
+    /// Convert this router into a [`MakeService`], that is a [`Service`] whose
     /// response is another service.
     ///
     /// This is useful when running your application with hyper's
@@ -357,6 +396,17 @@ where
 
         RouterFuture::from_oneshot(route.oneshot(req))
     }
+
+    fn panic_on_matchit_error(&self, err: matchit::InsertError) {
+        if self.nested_at_root {
+            panic!(
+                "Invalid route: {}. Note that `nest(\"/\", _)` conflicts with all routes. Use `Router::fallback` instead",
+                err,
+            );
+        } else {
+            panic!("Invalid route: {}", err);
+        }
+    }
 }
 
 impl<B> Service<Request<B>> for Router<B>
@@ -574,6 +624,6 @@ where
 
 #[test]
 fn traits() {
-    use crate::tests::*;
+    use crate::test_helpers::*;
     assert_send::<Router<()>>();
 }
